import {
  __async
} from "../chunk-P5H5YIIE.js";
function isPromise(value) {
  return typeof value === "object" && value !== null && typeof value.then === "function";
}
function observableToPromise(observable) {
  return new Promise((resolve, reject) => {
    const subscription = observable.subscribe({
      next(v) {
        resolve(v);
        subscription.unsubscribe();
      },
      error: reject,
      complete() {
        reject(new Error("no value resolved"));
      }
    });
  });
}
function isObservable(value) {
  return typeof value === "object" && value !== null && "subscribe" in value && typeof value.subscribe === "function";
}
function isAsyncIterable(input) {
  return typeof input === "object" && input !== null && // Some browsers still don't have Symbol.asyncIterator implemented (iOS Safari)
  // That means every custom AsyncIterable must be built using a AsyncGeneratorFunction (async function * () {})
  (input[Symbol.toStringTag] === "AsyncGenerator" || Symbol.asyncIterator in input);
}
function asyncIterableToPromise(input) {
  return __async(this, null, function* () {
    var _a;
    const iteratorReturn = (_a = ("return" in input ? input : input[Symbol.asyncIterator]()).return) == null ? void 0 : _a.bind(input);
    const iteratorNext = ("next" in input ? input : input[Symbol.asyncIterator]()).next.bind(input);
    const result = yield iteratorNext();
    void (iteratorReturn == null ? void 0 : iteratorReturn());
    return result.value;
  });
}
function fetcherReturnToPromise(fetcherResult) {
  return __async(this, null, function* () {
    const result = yield fetcherResult;
    if (isAsyncIterable(result)) {
      return asyncIterableToPromise(result);
    }
    if (isObservable(result)) {
      return observableToPromise(result);
    }
    return result;
  });
}
export {
  fetcherReturnToPromise,
  isAsyncIterable,
  isObservable,
  isPromise
};
