import "../chunk-P5H5YIIE.js";
import {
  createMultipartFetcher,
  createSimpleFetcher,
  isSubscriptionWithName,
  getWsFetcher
} from "./lib";
function createGraphiQLFetcher(options) {
  const httpFetch = options.fetch || typeof window !== "undefined" && window.fetch;
  if (!httpFetch) {
    throw new Error("No valid fetcher implementation available");
  }
  options.enableIncrementalDelivery = options.enableIncrementalDelivery !== false;
  const simpleFetcher = createSimpleFetcher(options, httpFetch);
  const httpFetcher = options.enableIncrementalDelivery ? createMultipartFetcher(options, httpFetch) : simpleFetcher;
  return (graphQLParams, fetcherOpts) => {
    if (graphQLParams.operationName === "IntrospectionQuery") {
      return (options.schemaFetcher || simpleFetcher)(
        graphQLParams,
        fetcherOpts
      );
    }
    const isSubscription = (fetcherOpts == null ? void 0 : fetcherOpts.documentAST) ? isSubscriptionWithName(
      fetcherOpts.documentAST,
      graphQLParams.operationName || void 0
    ) : false;
    if (isSubscription) {
      const wsFetcher = getWsFetcher(options, fetcherOpts);
      if (!wsFetcher) {
        throw new Error(
          `Your GraphiQL createFetcher is not properly configured for websocket subscriptions yet. ${options.subscriptionUrl ? `Provided URL ${options.subscriptionUrl} failed` : "Please provide subscriptionUrl, wsClient or legacyClient option first."}`
        );
      }
      return wsFetcher(graphQLParams);
    }
    return httpFetcher(graphQLParams, fetcherOpts);
  };
}
export {
  createGraphiQLFetcher
};
